@@ -1,18 +1,19 @@
-package Crypt::DSA;
+# $Id: DSA.pm,v 1.10 2001/03/24 01:15:50 btrott Exp $
 
-use 5.005;
+package Crypt::DSA;
 use strict;
+
+use Math::Pari qw( PARI );
+use Crypt::Random qw( makerandom );
 use Digest::SHA1 qw( sha1 );
 use Carp qw( croak );
+
 use Crypt::DSA::KeyChain;
 use Crypt::DSA::Key;
-use Crypt::DSA::Signature;
-use Crypt::DSA::Util qw( bitsize bin2mp mod_inverse mod_exp makerandom );
+use Crypt::DSA::Util qw( bitsize bin2mp mod_inverse mod_exp );
 
 use vars qw( $VERSION );
-BEGIN {
-    $VERSION = '0.15_01';
-}
+$VERSION = '0.02';
 
 sub new {
     my $class = shift;
@@ -53,10 +54,7 @@ sub sign {
     $s -= $key->q if $s > $key->q;
     $s = ($s * $key->kinv) % $key->q;
 
-    my $sig = Crypt::DSA::Signature->new;
-    $sig->r($key->r);
-    $sig->s($s);
-    $sig;
+    { r => $key->r, s => $s }
 }
 
 sub _sign_setup {
@@ -64,7 +62,7 @@ sub _sign_setup {
     my $key = shift;
     my($k, $r);
     {
-        $k = makerandom(Size => bitsize($key->q));
+        $k = makerandom(Size => bitsize($key->q), Strength => 0);
         $k -= $key->q if $k >= $key->q;
         redo if $k == 0;
     }
@@ -87,23 +85,20 @@ sub verify {
     }
     croak __PACKAGE__, "->verify: Need a Signature"
         unless $sig = $param{Signature};
-    my $u2 = mod_inverse($sig->s, $key->q);
+    my $u2 = mod_inverse($sig->{s}, $key->q);
     my $u1 = bin2mp($dgst);
     $u1 = ($u1 * $u2) % $key->q;
-    $u2 = ($sig->r * $u2) % $key->q;
+    $u2 = ($sig->{r} * $u2) % $key->q;
     my $t1 = mod_exp($key->g, $u1, $key->p);
     my $t2 = mod_exp($key->pub_key, $u2, $key->p);
     $u1 = ($t1 * $t2) % $key->p;
     $u1 %= $key->q;
-    $u1 == $sig->r;
+    $u1 == $sig->{r};
 }
 
 1;
-
 __END__
 
-=pod
-
 =head1 NAME
 
 Crypt::DSA - DSA Signatures and Key Generation
@@ -194,8 +189,8 @@ The default is 0.
 Signs a message (or the digest of a message) using the private
 portion of the DSA key and returns the signature.
 
-The return value--the signature--is a I<Crypt::DSA::Signature>
-object.
+The signature is a hash reference with two keys: I<s> and
+I<r>.
 
 I<%arg> can include:
 
@@ -249,7 +244,7 @@ This argument is required.
 =item * Signature
 
 The signature itself. Should be in the same format as returned
-from I<sign>, a I<Crypt::DSA::Signature> object.
+from I<sign>, a hash reference with two keys, I<s> and I<r>.
 
 This argument is required.
 
@@ -268,30 +263,13 @@ be created and used in the verification process.
 
 =back
 
-=head1 TODO
+=head1 AUTHOR & COPYRIGHT
 
-Add ability to munge format of keys. For example, read/write keys
-from/to key files (SSH key files, etc.), and also write them in
-other formats.
+Benjamin Trott, ben@rhumba.pair.com
 
-=head1 SUPPORT
-
-Bugs should be reported via the CPAN bug tracker at
-
-L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Crypt-DSA>
-
-For other issues, contact the author.
-
-=head1 AUTHOR
-
-Benjamin Trott E<lt>ben@sixapart.comE<gt>
-
-=head1 COPYRIGHT
-
-Except where otherwise noted,
-Crypt::DSA is Copyright 2006 - 2009 Benjamin Trott.
-
-Crypt::DSA is free software; you may redistribute it
-and/or modify it under the same terms as Perl itself.
+Except where otherwise noted, Crypt::DSA is Copyright 2001
+Benjamin Trott. All rights reserved. Crypt::DSA is free
+software; you may redistribute it and/or modify it under
+the same terms as Perl itself.
 
 =cut
