@@ -1,21 +1,15 @@
+# $Id: KeyChain.pm,v 1.5 2001/03/23 23:19:13 btrott Exp $
+
 package Crypt::DSA::KeyChain;
-
 use strict;
-use Math::BigInt try => 'GMP, Pari';
-use Digest::SHA1 qw( sha1 );
-use Carp qw( croak );
-use IPC::Open3;
-use File::Spec;
-use File::Which ();
-use Symbol qw( gensym );
 
-use vars qw{$VERSION};
-BEGIN {
-    $VERSION = '0.15_01';
-}
+use Math::Pari qw( PARI isprime );
+use Digest::SHA1 qw( sha1 );
+use Crypt::Random qw( makerandom );
+use Carp qw( croak );
 
 use Crypt::DSA::Key;
-use Crypt::DSA::Util qw( bin2mp bitsize mod_exp makerandom isprime );
+use Crypt::DSA::Util qw( bin2mp bitsize mod_exp );
 
 sub new {
     my $class = shift;
@@ -24,102 +18,64 @@ sub new {
 
 sub generate_params {
     my $keygen = shift;
-    my %param  = @_;
-    my $bits   = Math::BigInt->new($param{Size});
+    my %param = @_;
+
+    my $bits = PARI($param{Size});
     croak "Number of bits (Size) is too small" unless $bits;
     delete $param{Seed} if $param{Seed} && length $param{Seed} != 20;
     my $v = $param{Verbosity};
 
-    # try to use fast implementations found on the system, if available.
-    unless ($param{Seed} || wantarray || $param{PurePerl}) {
+    my($counter, $q, $p, $seed) = (0);
+    {
+        my($seedp1);
+        my $test = PARI(1); $test <<= ($bits-1);
 
-        # OpenSSL support
-        my $bin     = $^O eq 'MSWin32' ? 'openssl.exe' : 'openssl';
-        my $openssl = File::Which::which($bin);
-        if ( $openssl ) {
-            print STDERR "Using openssl\n" if $v;
-            my $bits_n = int($bits);
-            open( NULL, ">", File::Spec->devnull );
-            my $pid = open3( gensym, \*OPENSSL, ">&NULL", "$openssl dsaparam -text -noout $bits_n" );
-            my @res;
-            while( <OPENSSL> ) {
-                push @res, $_;
-            }
-            waitpid( $pid, 0 );
-            close OPENSSL;
-            close NULL;
+        {
+            print STDERR "." if $v;
+            $seed = $param{Seed} ? delete $param{Seed} :
+                join '', map chr rand 255, 1..20;
 
-            my %parts;
-            my $cur_part;
-            foreach (@res) {
-                if (/^\s+(\w):\s*$/) {
-                    $cur_part = $1;
-                    next;
-                }
-                if (/^\s*((?:[0-9a-f]{2,2}:?)+)\s*$/) {
-                    $parts{$cur_part} .= $1;
-                }
-            }
+            $seedp1 = _seed_plus_one($seed);
+            my $md = sha1($seed) ^ sha1($seedp1);
 
-            $parts{$_} =~ s/://g for keys %parts;
+            vec($md, 0, 8) |= 0x80;
+            vec($md, 19, 8) |= 0x01;
 
-            if (scalar keys %parts == 3) {
-                my $key = Crypt::DSA::Key->new;
-                $key->p(Math::BigInt->new("0x" . $parts{p}));
-                $key->q(Math::BigInt->new("0x" . $parts{q}));
-                $key->g(Math::BigInt->new("0x" . $parts{g}));
-                return $key;
-            }
+            $q = bin2mp($md);
+            redo unless isprime($q);
         }
 
-    }
+        print STDERR "*\n" if $v;
+        my $n = int(("$bits"-1) / 160);
+        my $b = ($bits-1)-PARI($n)*160;
 
-    # Pure Perl version:
+        {
+            print STDERR "." if $v;
+            my $W = PARI(0);
+            for my $k (0..$n) {
+                $seedp1 = _seed_plus_one($seedp1);
+                my $r0 = bin2mp(sha1($seedp1));
+                $r0 %= PARI(2) ** $b
+                    if $k == $n;
+                $W += $r0 << (PARI(160) * $k);
+            }
 
-    my($counter, $q, $p, $seed, $seedp1) = (0);
-
-    ## Generate q.
-    SCOPE: {
-        print STDERR "." if $v;
-        $seed = $param{Seed} ? delete $param{Seed} :
-            join '', map chr rand 256, 1..20;
-        $seedp1 = _seed_plus_one($seed);
-        my $md = sha1($seed) ^ sha1($seedp1);
-        vec($md, 0, 8) |= 0x80;
-        vec($md, 19, 8) |= 0x01;
-        $q = bin2mp($md);
-        redo unless isprime($q);
-    }
-
-    print STDERR "*\n" if $v;
-    my $n = int(("$bits"-1) / 160);
-    my $b = ($bits-1)-Math::BigInt->new($n)*160;
-    my $p_test = Math::BigInt->new(1); $p_test <<= ($bits-1);
-
-    ## Generate p.
-    SCOPE: {
-        print STDERR "." if $v;
-        my $W = Math::BigInt->new(0);
-        for my $k (0..$n) {
-            $seedp1 = _seed_plus_one($seedp1);
-            my $r0 = bin2mp(sha1($seedp1));
-            $r0 %= Math::BigInt->new(2) ** $b
-                if $k == $n;
-            $W += $r0 << (Math::BigInt->new(160) * $k);
+            my $X = $W + $test;
+            $p = $X - ($X % (2 * $q) - 1);
+            last if $p >= $test && isprime($p);
+            redo unless ++$counter >= 4096;
         }
-        my $X = $W + $p_test;
-        $p = $X - ($X % (2 * $q) - 1);
-        last if $p >= $p_test && isprime($p);
-        redo unless ++$counter >= 4096;
     }
 
     print STDERR "*" if $v;
     my $e = ($p - 1) / $q;
-    my $h = Math::BigInt->new(2);
+    my $h = PARI(2);
     my $g;
-    SCOPE: {
+    {
         $g = mod_exp($h, $e, $p);
-        $h++, redo if $g == 1;
+        last unless $g == 1;
+        $h++;
+        redo;
     }
     print STDERR "\n" if $v;
 
@@ -135,12 +91,13 @@ sub generate_keys {
     my $keygen = shift;
     my $key = shift;
     my($priv_key, $pub_key);
-    SCOPE: {
+    {
         my $i = bitsize($key->q);
-        $priv_key = makerandom(Size => $i);
+        $priv_key = makerandom(Size => $i, Strength => 0);
         $priv_key -= $key->q if $priv_key >= $key->q;
         redo if $priv_key == 0;
     }
+
     $pub_key = mod_exp($key->g, $priv_key, $key->p);
     $key->priv_key($priv_key);
     $key->pub_key($pub_key);
@@ -156,8 +113,7 @@ sub _seed_plus_one {
 }
 
 1;
-
-=pod
+__END__
 
 =head1 NAME
 
@@ -251,7 +207,7 @@ a I<Crypt::DSA::Key> object.
 
 =head1 AUTHOR & COPYRIGHT
 
-Please see the L<Crypt::DSA> manpage for author, copyright,
+Please see the Crypt::DSA manpage for author, copyright,
 and license information.
 
 =cut
